import unittest
from altius_l1pp_preflight.configuration import OpticsSpecs, PSFSpecs, AltiusConfigurationKey, Coords, WindowMappingType
from altius_l1pp_preflight.radiance_simulator import AltiusL1ppSimpleSkimpyInterface, SimulationRadianceDataBISA


# ------------------------------------------------------------------------------
#           AltiusL1ppTestConfiguration
# ------------------------------------------------------------------------------
class AltiusL1ppTestConfiguration(unittest.TestCase):

    def test_psf_vis_config(self):
        config = AltiusConfigurationKey(channel='vis', wavelength=550.0, datestr='2020-10-20')
        psfspecs = PSFSpecs(config)

    def test_detector_config(self):
        config = AltiusConfigurationKey(channel='vis', wavelength=550.0, datestr='2020-10-20')
        dspecs = OpticsSpecs(config)

    def test_mapping(self):
        config = AltiusConfigurationKey(channel='vis', wavelength=550.0, datestr='2020-10-20')
        optics = OpticsSpecs(config)
        location = Coords(x=8.2, y=251.2, windowtype=WindowMappingType.WINDOW_L1C)
        detector = optics.detector_specs.window_hierarchy.convert_to(location, WindowMappingType.WINDOW_DETECTOR)
        newlocation = optics.detector_specs.window_hierarchy.convert_to(detector, WindowMappingType.WINDOW_L1C)
        self.assertAlmostEqual(location.x, newlocation.x)
        self.assertAlmostEqual(location.y, newlocation.y)
        self.assertAlmostEqual(detector.x, 1230.0)
        self.assertAlmostEqual(detector.y, 1003.6)
        radians = optics.pixel_to_radians(location)

    def test_simple_skimpy_interface(self):
        configkey = AltiusConfigurationKey('vis')
        optics = OpticsSpecs(configkey)
        simulator = AltiusL1ppSimpleSkimpyInterface(optics, tp_latitude=38.0)
        radiance, latitude, longitude, altitude = simulator.radiance(550.0, albedo=1.0)
        self.assertAlmostEqual(latitude[750], 38.000077042437994, places=5)
        self.assertAlmostEqual(altitude[750], 50066.414668127894, places=5)
        self.assertAlmostEqual(radiance[750] / 541922109590.63934, 1.0, places=8)

    def test_bisa_stimuli(self):
        configkey = AltiusConfigurationKey('vis')
        optics = OpticsSpecs(configkey)
        stimuli = SimulationRadianceDataBISA()
        signal, error = stimuli.make_noof_image(optics, 3, 4)
        self.assertAlmostEqual(signal[3000, 3000], 6010.094451515401, places=5)
        self.assertAlmostEqual(signal[1000, 0], 27883.177782753806, places=5)


if __name__ == "__main__":
    tests = AltiusL1ppTestConfiguration()
    unittest.main()
